package meeting.business.modeldades;
import SappFramework.Interfaces.IUsuari;

/**
 * <li>Classe que modela les dades d'un usuari.</li>
 * <li>Implementa la interfíci IUsuari del SappFramework per poder utilitzar 
 * el sistema de validació de rols en les accions que implementin la anotació "Rol"</li>
 * @author Josep Jiménez
 */
public class Usuari implements IUsuari {
    int idusuari;
    String nom;
    String correu;
    String password;
    String sessionid;
    boolean validat;
    String rol;

    /**
     * Obtenir el correu de l'usuari. Serà l'identificador per poder validar una sessió
     * @return Retorna el correu de l'usuari
     */
    public String getCorreu() {
        return correu;
    }

    /**
     * Actualitzar el correu de l'usuari
     * @param correu Correu de l'usuari
     */
    public void setCorreu(String correu) {
        this.correu = correu;
    }

    /**
     * Obtenir l'identificador de l'usuari.
     * @return Retorna el identificador de l'usuari
     */
    public int getIdusuari() {
        return idusuari;
    }

    /**
     * Actualitzar l'identificador de l'usuari
     * @param idusuari Identificador de l'usuari
     */
    public void setIdusuari(int idusuari) {
        this.idusuari = idusuari;
    }

    /**
     * Obtenir el nom complert de l'usuari
     * @return Retorna el nom coplet de l'usuari
     */
    public String getNom() {
        return nom;
    }

    /**
     * Actualitzar el nom complert de l'usuari
     * @param nom Nom complert de l'usuari
     */
    public void setNom(String nom) {
        this.nom = nom;
    }

    /**
     * Obtenir el password de l'usuari
     * @return Retorna el password de l'usuari
     */
    public String getPassword() {
        return password;
    }

    /**
     * Actualitza el password de l'usuari
     * @param password Password de l'usuari
     */
    public void setPassword(String password) {
        this.password = password;
    }

    /**
     * Obtenir si l'usuri està validat en la sessió actual
     * @return True si l'usuari s'ha validad
     */
    public boolean getValidat() {
        return validat;
    }

    /**
     * Actualitzar l'estat de l'usuari per aquesta sessió
     * @param validat Amb true es valida.
     */
    public void setValidat(boolean validat) {
        this.validat = validat;
    }

    /**
     * Obtenir sessió per la qual està validat l'usuari
     * @return Sessió en la que s'ha validat l'usuari
     */
    public String getSessionId(){
        return this.sessionid;
    }

    /**
     * Actualitzar la sessió en la que s'ha validat l'usuari
     * @param sessionid Sessió en la que s'ha validat l'usuari
     */
    public void setSessionId(String sessionid){
        this.sessionid = sessionid;
    }
    
    /**
     * Obtenir el rol de l'usuari. Si una acció te l'anotació "Rol" es comprovarà que l'usuari tingui el mateix tipus de rol assignat
     * @return Retorna el rol assignat a l'usuari
     */
    public String getRol(){
        return this.rol;
    }

    /**
     * Actualitzar el rol de l'usuari. Si una acció te l'anotació "Rol" es comprovarà que l'usuari tingui el mateix tipus de rol assignat
     * @param rol Rol assignat a l'usuari
     */
    public void setRol(String rol){
        this.rol = rol;
    }
    
    
}
