/*
 * Controlador pels usuaris de l'aplicació
 */
package meeting.presentation.controladors;
import meeting.business.DB.DBdades;

import SappFramework.Annotation.*;
import SappFramework.Eines.Constants;

import java.util.logging.*;
import java.util.List;

/**
 * <li>Cotrolador per gestionar els comentaris de les quedades.</li>
 * <li>El nom a utilitzar les pàgines web serà "comentaris"</li>
 * <li>El tipus d'execució serà de petició resposta, per tant els seus resultats no es guardaran
 * a la sessió. S'han d'actualitzar cada vegada.</li>
 * @author Josep Jiménez
 */
@Controlador (nom="comentaris", tipusexecucio= Constants.TIPUS_EXECUCIO_PR )
public class ConComentaris {
    static Logger logger = Logger.getLogger(ConComentaris.class.getName());
    /** Identificador de la quedada a que corresponen els comentaris      */
    private int Idquedada;
    /** Comentari d'una quedada.  */
    private String comentari;
    /** Llista de comentaris d'una quedada      */
    private List llistacomentaris;
    /** 
     * Error produït per qualsevol acció. Es podrà llegir als JSP, o per l'atribut error d'aquesta classe o per l'atribut de sessió
     * amb el nom "error" els dos mètodes son vàlids
     */
    private String error;
    /**
     * Acció que retorna la llista de comentaris d'una quedada
     * @param Idquedada Identificador de la quedada de la qual es volen optenir els comentaris. El valor s'obté directament de la web a través del SappFramework.
     * @return Retornarà true hi ha error a la base de dades.
     */
    @Accio (nom="comentquedada")
    public boolean getLlistComentarisQuedada(@ParamWeb (nom ="Idquedada")int Idquedada){
       boolean result =true;
       DBdades db = new DBdades();
       try{
            this.llistacomentaris = db.comentarisByIdQuedada(Idquedada); 
       result =false;
       } catch (Exception e) {
           result = true;
       }
     return result;
   }
    /**
    *  Mètode de preexecució per controlar que un comentari a inserir no estigui buit. Aquest mètode s'executa abans del mètode definit a l'anotació.
     * @param comentari Comentari a inserir d'una quedada. El valor s'obté directament de la web a través del SappFramework.
     * @return Retornarà true hi ha error durant la insersió del comentari a la base de dades. Si hi ha error es dirigirà a la pàgina definida a l'anotació
     */
    @Preexecucio ( retornError="errorformat", metodeacio={"setComentarisQuedada", "1"})
    public boolean Ctrlcomentari(@ParamWeb (nom ="comentari")String comentari){
        boolean result =false;
        if (comentari.equals("")){
             error = "El comentari està buit.";
            result = true;
        }

        return result;
   }    
    /**
     * Acció que afegeix un nou comentari d'un usuari a una quedada
     * @param Idquedada Identificador de la quedada a la que s'afegeix el comentari
     * @param comentari Comentari a afegir a la quedada
     * @param usuari Usuari que realitza el comentari
     * @param quedada Objecte quedada que s'extreu d'un controlador de l'aplicació
     * @return Retorna el resultat de l'acció true = error i es SappFramework redirigirà a la pàgina d'error. 
     * false = ok es redigirrà a la pàgina que o de la que provenia si no està configurada
     */
    @Rol (rols={"user", "admin"})
    @Accio (nom="afegircomentari")
    public boolean setComentarisQuedada(@ParamWeb (nom ="Idquedada")int Idquedada,
                                        @ParamWeb (nom ="comentari")String comentari,
                                        @ParamApp (nom="usuari") ConUsuaris usuari,
                                        @ParamApp (nom="quedada") ConCuocdada quedada){
       boolean result =true;
       DBdades db = new DBdades();
       try{
            db.afegirComentari(Idquedada, usuari.getUsuari().getIdusuari(), usuari.getUsuari().getNom(), comentari);
            getLlistComentarisQuedada(Idquedada);
            quedada.setLlistacomentaris(llistacomentaris);
            quedada.convertirDades();
       result =false;
       } catch (Exception e) {
           result = true;
       }
     return result;
   }    
    
    /**
     * Obtenir l'identificador de la quedada en curs
     * @return Retorna el identificador de la quedada en curs
     */
    public int getIdquedada() {
        return Idquedada;
    }

    /**
     * Actualitzar el identificador de la quedada en curs
     * @param Idquedada Identificador de la quedada en curs
     */
    public void setIdquedada(int Idquedada) {
        this.Idquedada = Idquedada;
    }

    /**
     * Obtenir llista de comentaris de la quedada en curs
     * @return Llista de comentaris de la quedada en curs
     */
    public List getLlistacomentaris() {
        return llistacomentaris;
    }

    /**
     * Actualitzar llista de comentaris de la quedada en curs
     * @param llistacomentaris Llista de comentaris de la quedada en curs
     */
    public void setLlistacomentaris(List llistacomentaris) {
        this.llistacomentaris = llistacomentaris;
    }

    /**
     * Obtenir comentari de la quedada en curs
     * @return Comentari de la quedada en curs
     */
    public String getComentari() {
        return comentari;
    }

    /**
     * Actualitzar comentari de la quedada en curs
     * @param comentari Comentari de la quedada en curs
     */
    public void setComentari(String comentari) {
        this.comentari = comentari;
    }

    /**
     * Obtenir error actual
     * @return Error actual
     */
    public String getError() {
        return error;
    }

    /**
     * Actualitzar error actual
     * @param error Error actual
     */
    public void setError(String error) {
        this.error = error;
    }
    
}
