package meeting.presentation.controladors;
import meeting.business.modeldades.Cuocdada;
import meeting.business.DB.DBdades;

import SappFramework.Annotation.*;

import java.util.logging.*;
import java.util.List;
import java.util.Calendar;
import java.text.SimpleDateFormat;
import java.util.Iterator;

/**
 * <li>Controlador principal de gestió de les quedades.</li>
 * <li>No es defineix nom del controlador com a exemple per demostra el 
 * funcionament del sistema amb el nom de la classe del controlador.</li>
 * @author Josep Jiménez
 */
@Controlador
public class ConCuocdada {
    static Logger logger = Logger.getLogger(ConCuocdada.class.getName());
    private Cuocdada quedada;
    private int Idquedada;
    private List llistaquedades;
    private List llistacomentaris;
    private List llistausuarisquedada;
    private String datainici;
    private String datafinal;
    private String error;
    private String titol;
    private String descripcio;
    private String direccio;
    private String data;
    private String hora;

   /**
    * Acció que retorna una llista de les quedades que hi ha des de la data actual
    * @return Retorna el resultat de l'acció true = error i es SappFramework redirigirà a la pàgina d'error. 
    * false = ok es redigirrà a la pàgina que o de la que provenia si no està configurada
    */  
    @Accio (nom="properesquedades", retorn = "quedades")
    public boolean getLlistapropoeres(){
       boolean result =true;
       DBdades db = new DBdades();
       try{
       Calendar datahoraactual = Calendar.getInstance();
      String dataactual = (datahoraactual.get(Calendar.YEAR)+"/"+
                           (datahoraactual.get(Calendar.MONTH)+1)+"/"+
                            datahoraactual.get(Calendar.DAY_OF_MONTH));
       
       this.llistaquedades = db.quedadesByData(dataactual); 
       result =false;
       } catch (Exception e) {
           result = true;
       }

     return result;
   }

   /**
    * Acció que retorna la llista de totes les quedades de la BD
    * @return Retorna el resultat de l'acció true = error i es SappFramework redirigirà a la pàgina d'error. 
    * false = ok es redigirrà a la pàgina configurada o de la que provenia si no està configurada
    */  
    @Accio (nom="toteslesquedades", retorn = "totesquedades")
    public boolean getTotesquedades(){
       boolean result =true;
       DBdades db = new DBdades();
       try{
       this.llistaquedades = db.allQuedades();
       result =false;
       } catch (Exception e) {
           result = true;
       }

     return result;
   }
   
   /**
    * Mètode de preexecució Controla que la data d'inici estigui ben formada
     * @param datainici Data inici
     * @return true = error la data no es correcta interrupeix l'acció i rediregeix a la pàgina d'error, false = ok la data és correcta i continua l'execució 
    */  
   @Preexecucio ( retornError="errorformat", metodeacio={"getQuedadesPerData", "1", "afegirNovaQuedada", "1"})
   public boolean controlDataInici (@ParamWeb (nom ="datainici")String datainici){
       boolean result =true;
       SimpleDateFormat data = new SimpleDateFormat("dd/MM/yyyy");
       data.setLenient(false);
       try{
           data.parse(datainici);
           result =false;
       }catch (Exception e){
              error = "La data inicial no es correcta el format ha de ser dia/mes/any (dd/MM/yyy), comprovar que els valors siguin correctes";
              result =true;
       }
       return result; 
   }
   
    /**
    * Mètode de preexecució Controla que la data final estigui ben formada
    * @param datafinal Data final
    * @return true = error la data no es correcta interrupeix l'acció i rediregeix a la pàgina d'error, false = ok la data és correcta i continua l'execució 
    */  
   @Preexecucio ( retornError="errorformat", metodeacio={"getQuedadesPerData", "2"})
   public boolean controlDataFinal (@ParamWeb (nom ="datafinal")String datafinal){
       boolean result =true;
       SimpleDateFormat data = new SimpleDateFormat("dd/MM/yyyy");
       data.setLenient(false);
       try{
           data.parse(datafinal);
           result =false;
       }catch (Exception e){
              error = "La data final no es correcta el format ha de ser dia/mes/any (dd/MM/yyy), comprovar que els valors siguin correctes";
              result =true;
       }
       return result; 
   }
    
    /**
    * Mètode de preexecució Controla que la data d'inici sigui anterior o igual a la data final
    * @param datainici Data inicial
    * @param datafinal Data final
    * @return true = error l'interval de date no es correcte interrupeix l'acció i rediregeix a la pàgina d'error, false = ok l'interval és correcte i continua l'execució 
    */  
   @Preexecucio ( retornError="errorformat", metodeacio={"getQuedadesPerData", "3"})
   public boolean controlDates (@ParamWeb (nom ="datainici")String datainici,
                                @ParamWeb (nom ="datafinal")String datafinal){
       boolean result =true;
       SimpleDateFormat data = new SimpleDateFormat("dd/MM/yyyy");
       data.setLenient(false);
       try{
           data.parse(datainici);
           data.parse(datafinal);
           long init = data.parse(datainici).getTime();
           long fit =data.parse(datafinal).getTime();
           result =false;
           if(init > fit){
              error = "La data no es correcta la data inicial ha de ser anterior a la data final"; 
              result =true;
           }
       }catch (Exception e){
              error = "La data no es correcta el format ha de ser dia/mes/any (dd/MM/yyy), comprovar que els valors siguin correctes";
              result =true;
       }
       return result; 
   }   
      
    /**
    * Acció per obtenir una llista de quedades entre dues dates
    * @param datainici Data inicial de la cerca
    * @param datafinal Data final de la cerca
    * @return Retorna el resultat de l'acció true = error i el SappFramework redirigirà a la pàgina d'error. 
    * false = ok es redigirrà a la pàgina configurada o de la que provenia si no està configurada
    */    
    @Accio (nom="quedadesperdata", retorn="quedadesperdata")
    public boolean getQuedadesPerData(@ParamWeb (nom ="datainici")String datainici,
                                      @ParamWeb (nom ="datafinal")String datafinal){
       boolean result =true;
       DBdades db = new DBdades();
       try{
       this.llistaquedades = db.quedadesBetweenData(datainici, datafinal);
       result =false;
       } catch (Exception e) {
           result = true;
       }

     return result;
   }   
    
    /**
    * Mètode de postexecució per per mostrar les dates i l'hora de forma correcta una llista de quedades.
    * @return Retorna el resultat de postexecució true = error i el SappFramework redirigirà a la pàgina d'error. 
    * false = ok es redigirrà a la pàgina configurada a l'acció de la que prové si lo tenia configurada  rediregeix a la pàgina de la que provenia
    */        
   @Postexecucio ( metodeacio={ "getTotesquedades","1",
                                "getQuedadesPerData","1",
                                "getLlistapropoeres","1"})
   public boolean convertirDades(){
      //Crear iterador
      Iterator it = this.llistaquedades.iterator();
      
      while (it.hasNext()){
          //Recuperar quedada
          Cuocdada quedada = (Cuocdada) it.next();
          
          //Treure els segons de l'hora
          quedada.setHora(quedada.getHora().substring(0,5));
 
          //Recolocar l'any
          String any = quedada.getData().substring(0,4);
          String mes = quedada.getData().substring(5,7);
          String dia = quedada.getData().substring(8);
          quedada.setData(dia+"/"+mes+"/"+any);
          
      }
       return false;
   }
   
   /**
    * Acció que cerca una quedada per la seva id, també obté els comentaris asociats a la quedada de la cerca
    * @param Idquedada identificador de la quedada
    * @param commts Objecte controlador de comentaris.
    * @return Retorna el resultat de l'acció true = error i el SappFramework redirigirà a la pàgina d'error. 
    * false = ok es redigirrà a la pàgina configurada o de la que provenia si no està configurada
    */  
    @Accio (nom= "quedada", retorn = "quedada")
    public boolean getQuedada(@ParamWeb (nom ="Idquedada")int Idquedada, 
                              @ParamApp (nom="comentaris") ConComentaris commts){
       boolean result =true;
       DBdades db = new DBdades();
       try{
       this.quedada = db.quedadaById(Idquedada);
       commts.getLlistComentarisQuedada(Idquedada);
       llistacomentaris=commts.getLlistacomentaris();
       result =false;
       } catch (Exception e) {
           result = true;
       }
       return result;
   }
    
    /**
    * Mètode de postexecució per per mostrar la data i l'hora de forma correcta de una sóla quedada
    * @return Retorna el resultat de postexecució true = error i el SappFramework redirigirà a la pàgina d'error. 
    * false = ok es redigirrà a la pàgina configurada a l'acció de la que prové si lo tenia configurada  rediregeix a la pàgina de la que provenia
    */        
   @Postexecucio ( metodeacio={ "getQuedada","1"})
   public boolean convertirDada(){

      //Treure els segons de l'hora
      this.quedada.setHora(this.quedada.getHora().substring(0,5));

      //Recolocar l'any
      String any = this.quedada.getData().substring(0,4);
      String mes = this.quedada.getData().substring(5,7);
      String dia = this.quedada.getData().substring(8);
      this.quedada.setData(dia+"/"+mes+"/"+any);

      return false;
   }

    /**
    * Acció que retornar la llista d'usuaris d'una quedada
    * @return Retorna el resultat de l'acció true = error i el SappFramework redirigirà a la pàgina d'error. 
    * false = ok es redigirrà a la pàgina configurada o de la que provenia si no està configurada
    */
    @Rol (rols ={"user", "admin"})
    @Accio (nom="quiva", retorn="quiva")
    public boolean quiVa(){
        boolean result =true;
        DBdades db = new DBdades();
        llistausuarisquedada = db.quiVa(this.quedada.getIdquedada());
        result =false;
        return result;
   }
   
    /**
    * Acció per afegir un usuari a una quedada
     * @param usuari 
     * @return Retorna el resultat de l'acció true = error i el SappFramework redirigirà a la pàgina d'error. 
    * false = ok es redigirrà a la pàgina configurada o de la que provenia si no està configurada
    */
    @Rol (rols ={"user", "admin"})
    @Accio (nom="anarquedada", retorn="quiva")
    public boolean anar(@ParamApp (nom="usuari") ConUsuaris usuari){
        boolean result =true;
        DBdades db = new DBdades();
        result = db.afegirUsuariAQuedada(this.Idquedada, usuari.getUsuari().getIdusuari(), usuari.getUsuari().getNom());
        //Actualitzar llista de qui va
        quiVa();
        if(result){
            error = "No s'ha pogut apuntar l'usuari a la quedada. Comprova que no estiguis apuntat";
        }
        return result;
   }
    /**
    * Acció per controlar l'accés de l'usuari l'administrador
    * @return Retorna el resultat de l'acció true = error i el SappFramework redirigirà a la pàgina d'error. 
    * false = ok es redigirrà a la pàgina configurada o de la que provenia si no està configurada
    */
    @Rol (rols ={"admin"})
    @Accio (nom="selecadminafegirquedada", retorn="afegirquedada")
    public boolean selecAdmin(){
        return false;
   }
    
   /**
    * Mètode de preexecució per controlar que hi hagi dades a tots els camps de la quedada
    * @param  titol Titol de la quedada
    * @param  descripcio Descripció de la quedada
    * @param  direccio Direcció de la quedada
    * @return true = error algún camp està buit, interrupeix l'acció i rediregeix a la pàgina d'error, false = ok els camps estan tots cumplimentats i continua l'execució 
    */
    @Preexecucio ( retornError="errorformat", metodeacio={"afegirNovaQuedada", "2"})
    public boolean CtrlDadesNovaQuedada(@ParamWeb (nom ="titol")String titol,
                                        @ParamWeb (nom ="descripcio")String descripcio,
                                        @ParamWeb (nom ="direccio")String direccio){
        boolean result =true;
        if (titol.equals("") || descripcio.equals("") || direccio.equals("")){
            result =true;
             error = "És necessari omplir tots els camps";
        }else{
           result =false;
        }
        return result;
   }   
    
    
    /**
    * Mètode de preexecució per controlar que el format de l'hora es correcte
     * @param hora Hora de la quedada
     * @return true = error el format de l'hora no és correcte, interrupeix l'acció i rediregeix a la pàgina d'error, false = ok l'hora és i continua l'execució 
    */
    @Preexecucio ( retornError="errorformat", metodeacio={"afegirNovaQuedada", "3"})
    public boolean CtrlFormatHora(@ParamWeb (nom ="hora")String hora){
        boolean result =true;
        
       SimpleDateFormat dhora = new SimpleDateFormat("HH:mm");
       dhora.setLenient(false);
       result = false;
       try{
           dhora.parse(hora); 
        }catch(Exception e){
            result = true;
            error = "El format de l'hora no es correcta ha de ser HH:MM, de les 00 a les 23 hores, y de 00 a 59 minuts";
        }
        return result;
   }    
    
    
    /**
    * Acció per afegir una nova Cuocdada
    * @param titol Títol de la quedada
    * @param descripcio Descripció de la quedada
    * @param direccio Direcció on es realitzarà la quedada
    * @param datainici Data en que es realitzarà la quedada
    * @param hora Hora en que es realitzarà la quedada
    * @param usuari Usuari que crea la nova quedada
    * @return Retorna el resultat de l'acció true = error i el SappFramework redirigirà a la pàgina d'error. 
    * false = ok es redigirrà a la pàgina configurada o de la que provenia si no està configurada
    */
    @Rol (rols ={"admin"})
    @Accio (nom="afegirquedada", retorn="afegirquedada_ok")
    public boolean afegirNovaQuedada(@ParamWeb (nom ="titol")String titol,
                                     @ParamWeb (nom ="descripcio")String descripcio,
                                     @ParamWeb (nom ="direccio")String direccio,
                                     @ParamWeb (nom ="datainici")String datainici,
                                     @ParamWeb (nom ="hora")String hora,
                                     @ParamApp (nom="usuari") ConUsuaris usuari){
        boolean result =true;
        DBdades db = new DBdades();
        result = db.afegirQuocdada(usuari.getUsuari().getIdusuari(), titol, direccio, datainici, hora, descripcio);
        if(result){
            error = "Existeixen caracters no permesos per l'entrada de dades. Comprovar el text entrat";
            this.getTotesquedades();
        }
        return result;
   }

    /**
     * Obtenir la data
     * @return Retorna la data 
     */
    public String getData() {
        return data;
    }

    /**
     * Actualitzar data
     * @param data Data
     */
    public void setData(String data) {
        this.data = data;
    }

    /**
     * Obtenir la descripció d'una quedada
     * @return Retorna descripció d'una quedada
     */
    public String getDescripcio() {
        return descripcio;
    }

    /**
     * Actualitzar la descripció d'una quedada 
     * @param descripcio Descripció d'una quedada
     */
    public void setDescripcio(String descripcio) {
        this.descripcio = descripcio;
    }

    /**
     * Obtenir la direcció d'una quedada
     * @return Retorna la direcció d'una quedada
     */
    public String getDireccio() {
        return direccio;
    }

    /**
     * Actualitzar direcció d'una quedada
     * @param direccio Direcció d'una quedada
     */
    public void setDireccio(String direccio) {
        this.direccio = direccio;
    }

    /**
     * Obtenir l'hora d'una quedada
     * @return Retorna l'hora d'una quedada
     */
    public String getHora() {
        return hora;
    }

    /**
     * Actualitzar l'hora d'una quedada
     * @param hora Hora d'una quedada
     */
    public void setHora(String hora) {
        this.hora = hora;
    }

    /**
     * Obtebir títol d'una quedada
     * @return Títol d'una quedada
     */
    public String getTitol() {
        return titol;
    }

    /**
     * Actualitzar títol d'una quedada
     * @param titol Títol dpuna quedada
     */
    public void setTitol(String titol) {
        this.titol = titol;
    }
    
    /**
     * Obtebir la llista de comentaris de la quedada en curs
     * @return Retorna la llista de comentaris de la quedada en curs
     */
    public List getLlistacomentaris() {
        return llistacomentaris;
    }

    /**
     * Actualitzar la llista de comentaris de la quedada en curs
     * @param llistacomentaris Llista de comentaris de la quedada en curs
     */
    public void setLlistacomentaris(List llistacomentaris) {
        this.llistacomentaris = llistacomentaris;
    }

    /**
     * Obtenir llista de quedades del controlador
     * @return Llista de quedades del controlador
     */
    public List getLlistaquedades() {
        return llistaquedades;
    }

    /**
     * Actualitzar llista de quedades del controladors
     * @param llistaquedades LLista de quedades del controlador
     */
    public void setLlistaquedades(List llistaquedades) {
        this.llistaquedades = llistaquedades;
    }
 
    /**
     * Obtenir identificador d'una quedada
     * @return Retorna l'identificador d'una quedada
     */
    public int getIdquedada() {
        return Idquedada;
    }

    /**
     * Actualitzar l'identificador d'una quedada
     * @param Idquedada Identificador d'una quedada
     */
    public void setIdquedada(int Idquedada) {
        this.Idquedada = Idquedada;
    }

    /**
     * Obtenir llista de quedades del controlador
     * @return Llista de quedades del controlador
     */
    public List getLlistaQuedades() {
        return llistaquedades;
    }

    /**
     * Actualitzar llista de quedades del controladors
     * @param llistaquedades LLista de quedades del controlador
     */
    public void setLlistaQuedades(List llistaquedades) {
        this.llistaquedades = llistaquedades;
    }

    /**
     * Obtenir la quedada en curs
     * @return Retorna una quedada en curs
     */
    public Cuocdada getQuedada() {
        return quedada;
    }

    /**
     * Actualitzar la quedada en curs
     * @param quedada Quedada en curs
     */
    public void setQuedada(Cuocdada quedada) {
        this.quedada = quedada;
    }
   
    /**
     * Obtenir error actual
     * @return Error actual
     */
    public String getError() {
        return error;
    }

    /**
     * Actualitzar error actual
     * @param error Error actual
     */
    public void setError(String error) {
        this.error = error;
    }

    /**
     * Obtebir una data final
     * @return Retorna una data final
     */
    public String getDatafinal() {
        return datafinal;
    }

    /**
     * Actualitzar una data final
     * @param datafinal Data final
     */
    public void setDatafinal(String datafinal) {
        this.datafinal = datafinal;
    }

    /**
     * Obtenir una data incial
     * @return Retorna una data inicial
     */
    public String getDatainici() {
        return datainici;
    }

    /**
     * Actualitzar una data inicial
     * @param datainici Data inicial
     */
    public void setDatainici(String datainici) {
        this.datainici = datainici;
    }

    /**
     * Obtenir la llista d'usuaris que assiteixen a la quedada en curs
     * @return Llista d'usuaris que assiteixen a la quedada en curs
     */
    public List getLlistausuarisquedada() {
        return llistausuarisquedada;
    }

    /**
     * Actualitzar llista d'usuaris que assiteixen a la quedada en curs 
     * @param llistausuarisquedada Llista d'usuaris que assiteixen a la quedada en curs
     */
    public void setLlistausuarisquedada(List llistausuarisquedada) {
        this.llistausuarisquedada = llistausuarisquedada;
    }

}
