/*
 * Controlador pels usuaris de l'aplicació
 */
package meeting.presentation.controladors;
import meeting.business.modeldades.Usuari;
import meeting.business.DB.DBdades;

import SappFramework.Annotation.*;
import SappFramework.Interfaces.IValidacio;
import SappFramework.Interfaces.IUsuari;

import javax.servlet.http.HttpServletRequest;
import java.util.logging.*;

/**
 * 
 * @author Josep Jiménez
 */
@ControlUsuari
@Controlador (nom="usuaris")
public class ConUsuaris implements IValidacio {
    static Logger logger = Logger.getLogger(ConUsuaris.class.getName());
    private Usuari usuari;
    private String correu;
    private String password;
    private String session;
    private String error;

    /**
     * <li>Acció especial per validar un usuari en una sessió. Aquest mètode també és età gestionat pel SappFramework de forma especial.</li>
     * <li>Aquesta acció és l'unica que no retorna un booleà. En aquest cas es retorna l'usuari que s'està validant o null si no el troba.</li>
     * @param correu Correu que identifica un usuari introduït a la web
     * @param password Password de l'usuari introduit a la web
     * @param rec Request actual per poder extreure la sessió
     * @return Retorna un objecte amb la interfície IUsauri del SappFramework o null si no el troba. 
     */
    @Accio (nom="validasession", retornError="errorvalidacio", retorn = "welcome")
    public IUsuari validarSessio(@ParamWeb (nom ="correu")String correu,
                                 @ParamWeb (nom ="password") String password,
                                  @ParamRequest HttpServletRequest rec){
       DBdades db = new DBdades();
       Usuari usr = new Usuari();
       usr = db.cercaUsuariPerCorreu(correu);
       
       //Comprovar si l'usuari és correcte
       try{
           if (usr.getCorreu().equals(correu) && usr.getCorreu()!=null ){
               //Comprovar si el password es correcte
               if (usr.getPassword().equals(password) && usr.getPassword()!=null ){
                   //Validar la sessió
                   session=rec.getSession().getId();
                   usr.setValidat(true);
                   //borrem el pasword per que no es vegi
                   this.password ="";
                   usr.setPassword("");
                   
               }else{
                   error = "Password incorrecte";
                   usr.setRol("");
               }

           } else{
               error = "Usuari no existeix";
               usr.setRol("");
           }
       } catch(NullPointerException n){
            error = "Usuari o password incorrectes";
            usr.setRol("");
       }

     usr.setPassword("");
     this.usuari = usr;
     return usr;
    }
   /**
    * Obtenir el correu de l'usuari
    * @return Retona correu de l'usuari
    */
   public String getCorreu() {
        return correu;
    }

   /**
    * Actualitza correu de l'usuari
    * @param correu Correu de l'usuari
    */
   public void setCorreu(String correu) {
        this.correu = correu;
    }   
   
   /**
    * Obtebir error actual
    * @return Retorna l'error actual
    */
   public String getError() {
        return error;
    }

    /**
     * Actualitzar error actual
     * @param error Error actual
     */
    public void setError(String error) {
        this.error = error;
    }

    /**
     * Obtenir password de l'usuari
     * @return Retorna el password de l'usuari
     */
    public String getPassword() {
        return password;
    }

    /**
     * Actualitzar password de l'usuari
     * @param password Pasword de l'usuari
     */
    public void setPassword(String password) {
        this.password = password;
    }
    /**
     * Obtenir l'objecte de la classe Usuari
     * @return Retorna un objecte de la classe Usuari
     */
    public Usuari getUsuari() {
        return usuari;
    }

    /**
     * Actualitzar un objecte de la classe Usuari
     * @param usuari Objecte de la classe Usuari
     */
    public void setUsuari(Usuari usuari) {
        this.usuari = usuari;
    }   

    /**
     * Obtenir l'identificador de la sessió en que l'usuari es valida o està validat
     * @return Retorna l'identificador de la sessió en que l'usuari es valida o està validat
     */
    public String getSession() {
        return session;
    }

    /**
     * Actualitzar l'identificador de la sessió en que l'usuari es valida o està validat
     * @param session Identificador de la sessió en que l'usuari es valida o està validat
     */
    public void setSession(String session) {
        this.session = session;
    }
}
