package SappFramework.Annotation;

import java.lang.annotation.ElementType;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;

/**
 * <li> Annotació per definir que un mètode esdevendrà un l'acció a portar a terme amb enllaç a la web.</li>
 * <li> Els mètodes definits com accions sempre tenen un valor boleà de retorn </li>
 * <li> Si el valor de retorn és true es que l'acció acaba amb error i es redirigirà a la pàgina definida al paràmetre retorn(), si aquest paràmetre no està definit es redirigirà a la pàgina configurada al SappFramework com a pàgina d'error general</li>
 * <li> Si el valor de retorn és false es que l'acció acaba correctamet i es dirigirà a la pàgina definida al paràmetre retornError(), si aquest parèmetre no està definit es redirigirà a la pàgina d'on provenia l'acció</li>
 * @author Josep Jiménez
 */
@Retention(RetentionPolicy.RUNTIME)
@Target(ElementType.METHOD)
public @interface Accio {
     /**
    * <li> Nom de l'acció que s'utilitzarà a la web.</li> 
    * <li> Si no es defineix aquest parèmtre agafarà per defecte el nom del mètode en minúscules.</li>
    * <li> Si s'asigna un valor agafarà aquest com a nom de l'acció.</li>
    */
    String nom() default "";
    /**
    * <li> Definició de la pàgina jsp de retorn.</li>  
    * <li> Si l'acció és correcta o sigui que el booleà de retorn és false, es redirigirà a aquesta pàgina, no s'ha d'introduir l'extensió ".jsp",l'afegirà el SappFramework.</li> 
    * <li> Si no s'introdueix aquest paràmetre i l'acció és correcta, retornarà a la pàgina de la que venia l'acció.</li>
    */
    String retorn() default "";     
    /**
    * <li> Definició de la pàgina de retorn en cas d'error.</li>
    * <li> Si l'acció no és correcta o sigui que el booleà de retorn és true, es dirigirà a aquesta pàgina. </li>
    * <li> Si no es defineix aquest paràmetre el SappFramework retornarà la pàgina definida com a error general de las seva configuració.</li>
    */
    String retornError() default "";  
    
}