package SappFramework.Annotation;

import java.lang.annotation.ElementType;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;

import SappFramework.Eines.Constants;
/**
 * <li> Annotació per definir que una classe implementarà un controlador del framework.</li>
 * <li> Per utilitzar les accions i accedir als atributs d'un controlador, s'ha de fer a través de l'atribut "Sapp" que es pasarà a la pàgina Web. </li>
 * <li> L'atribut "Sapp" correspon a un "HashMap" on els identificadors son el noms del controladors que s'hagin configurat a l'apliació.</li>
 * <li> Per poder accedir a un controlador previament ha d'haber estat creat, a través de la crida d'una acció que incorpori, o també en el moment de ser cridat a través d'un paràmetre d'un mètode del tipus acció.</li>
 * @author Josep Jiménez
 */
@Retention(RetentionPolicy.RUNTIME)
@Target(ElementType.TYPE)
public @interface Controlador {
    
    /**
    * <li> Nom del controlador que s'utilitzarà a les pàgines web.</li>
    * <li> Si no es defineix agafa per defecte el nom de la classe en minúscules.</li>
    * <li> Si s'asigna un valor agafarà aquest com a nom del controlador.</li>
    */
    String nom() default "";  // nom del contenidor
        /**
    * <li> Tipus d'excució.</li>
    * <li> Pot ser TIPUS_EXECUCIO_SESSIO per determinar que el cicle de vida és de sessió, dura durant tota la sessió del usuari.</li>
    * <li> Pot ser TIPUS_EXECUCIO_PR de petició/resposta, només s'envia al web no es guarda a la sessió d'usuari.</li>
    */
    String tipusexecucio() default Constants.TIPUS_EXECUCIO_SESSIO;   
}