package SappFramework.Annotation;

import java.lang.annotation.ElementType;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;

/**
 * <li> Annotació per definir un mètode de postexecució el qual s'excutarà deprés de l'execució del mètode d'una acció.</li> 
 * <li> Les Postsexecucions sempre tenen un valor boleà de retorn.</li> 
 * <li> Si el valor de retorn és true és que la postexecució acaba amb error i es redirigirà a la pàgina d'error.</li> 
 * <li> Si el valor de retorn és false és que la postexecució acaba correctamet i es dirigirà a la pàgina definida com a sortida o de la que venia.</li> 
 * @author Josep Jiménez
 */
@Retention(RetentionPolicy.RUNTIME)
@Target(ElementType.METHOD)
public @interface Postexecucio {
     /**
    * <li> Nom del mètedo de l'acció després de la qual s'ha d'executar el mètode de postexecució.</li>
    * <li> Per definir la crida a la Postexecucio calen 2 paràmetres. </li>
    * <li> El primer paràmtre correspon amb el nom real del mètode de l'acció que del postprocessat, NO el seu alies.</li>
    * <li> El segon paràmtre ha de ser un número en format String que detalla l'ordre d'execució en que s'han d'anar executant les diferents postexecucions que pot tenir una acció.</li>
    * <li> Exemple:    @Postexecucio ( metodeacio={ "getTotesquedades","1",
    *                                          "getQuedadesPerData","1",
    *                                          "getLlistapropoeres","2"})</li>
    */
      String[] metodeacio();
    /**
    * <li> Definició de la pàgina de retorn en cas d'error. No s'ha d'introduir l'extensió ".jsp".</li> 
    * <li> Si no es defineix retornarà la pàgina definida com a error general al SappFramework.</li> 
    */
    String retornError() default "";  
    
}