package SappFramework.Annotation;

import java.lang.annotation.ElementType;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;

/**
 * <li> Annotació per definir un mètode de preexecució el qual s'excutarà abans de l'execució del mètode d'una acció.</li> 
 * <li> Les preexecucions sempre tenen un valor boleà de retorn.</li> 
 * <li> Si el valor de retorn es false és que la preexecució acaba amb error i es redirigirà a la pàgina d'error.</li> 
 * <li> Si el valor de retorn es true és que la preexecució acaba correctamet i es continuarà mab l'acció definida.</li> 
 * @author Josep Jiménez
 */
@Retention(RetentionPolicy.RUNTIME)
@Target(ElementType.METHOD)
public @interface Preexecucio {
     /**
    * <li> Nom del mètedo de l'acció que s'ha de validar.</li> 
    * <li> Per definir la crida a la prexecució cales 2 paràmetres.</li>  
    * <li> el primer correspon amb el nom real del mètode de l'acció que del preprocessat, NO el seu alies. </li> 
    * <li> el segon ha de ser un nombre en format String que detalla l'ordre d'execució en que s'han d'anar executant les diferents preexecucions que pot tenir una acció.</li> 
    * <li> Exemple:    @Preexecucio ( metodeacio={ "getTotesquedades","1",
    *                                          "getQuedadesPerData","1",
    *                                          "getLlistapropoeres","2"})</li> 
    */
     String[] metodeacio();
    /**
    * <li> Definició de la pàgina de retorn en cas d'error. No s'ha d'introduir l'extensió ".jsp".</li> 
    * <li> Si no es defineix retornarà la pàgina definida com a error general al SappFramework.</li> 
    */
    String retornError() default "";  
    
}