package SappFramework.Frontend;

import java.io.IOException;

import javax.servlet.RequestDispatcher;
import javax.servlet.ServletException;
import javax.servlet.ServletConfig;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import java.util.logging.*;
import java.util.Enumeration;
import java.util.Set;

import SappFramework.Annotation.*;
import SappFramework.Gestio.*;
import SappFramework.Eines.*;
import SappFramework.Conversors.*;
import SappFramework.Interfaces.*;

import org.reflections.scanners.TypeAnnotationsScanner;
import org.reflections.ReflectionsException;
import org.reflections.Reflections;
import java.util.Map;
import java.util.Vector;
import java.util.Collection;
import java.util.List;
import java.util.Iterator;
import java.util.HashMap;
import java.lang.annotation.Annotation;
import java.lang.String;
import java.lang.reflect.Array;
import java.lang.reflect.Method;
import java.lang.reflect.Modifier;
import java.lang.reflect.ParameterizedType;
import java.lang.reflect.Type;
import java.lang.reflect.InvocationTargetException;
import java.lang.IllegalAccessException;
import java.lang.reflect.InvocationTargetException;
import  java.lang.IllegalAccessException;
import  java.lang.ClassNotFoundException;
import java.lang.InstantiationException;



/**
 * Classe que implementa el frontend per gestionar les peticions que s'envien des de la web.
 * @author Josep Jiménez
 */
public class ServletFrontController extends HttpServlet {
     static Logger logger = Logger.getLogger(ServletFrontController.class.getName());
     /** Paràmetre d'inicialització a web.xml indica la ruta base de l'aplicació configurada.*/
     private static final String RUTA_APLICACIO = "ruta.base";
     /** Paràmetre d'inicialització a web.xml indica la pàgina d'error configurada.*/
     private static final String PAGE_ERROR = "default.page.error"; 
     /** Paràmetre d'inicialització a web.xml indica la pàgina d'error de privilegis configurada.*/     
     private static final String PAGE_ERROR_PRIVILEGIS = "page.error.privilegis";
     /** Identificació de l'última pàgina que s'ha accedit satisfactoriament.*/     
     private String ultimapagina ="";
     /** Control de la seqüencia d'erros.*/ 
     private boolean errors =false;
     /** Pàgina d'error general configurada.*/ 
     private String paginaError="";
      /** Pàgina d'error de privilegis configurada.*/     
     private String paginaErrorPrivilegis="";
      /** Ruta de l'aplicació configurada.*/     
     private String rutaApp= "";
     /** Mapa de nom de controladors que s'han trobat a l'aplicació i estan disponibles per poder ser accedits.*/ 
     private Map <String, String> mapacontroladors = new HashMap<String, String>();
     /** Relació entre el nom real "el de la classe" i el nom definit al paràmtre "nom" del controlados.*/ 
     private Map <String, String> mapacontroladorsnomreal = new HashMap<String, String>();
      /** Mapa nom d'accions que s'han trobat a l'aplicació i estan disponibles per poder ser accedits.*/    
     private Map <String, String> mapaaccions = new HashMap<String, String>();
     /** Mapa de controladors amb el seu nom com a paràmetre de recuperaicó del mapa i l'objecte controlador com a element a recuperar.*/    
     private Map <String, Ccontrolador> llistaControladors = new HashMap<String, Ccontrolador>();
     /** Mapa de sessions amb l'identificador com a parèmtre de recuperació del mapa i un mapa dde controladors com a element a recuperar.*/    
     private Map <String, HashMap> mapaSessions = new HashMap<String, HashMap>();
     /** Mapa d¡elements de sortida que s'enviarà al web, aquest mapa s'empre serà accecible des del Web amb el nom "Sapp" .*/ 
     private Map <String, Object> sortidadesdesessio;
     
     
     /** Constructor de la classe hereta de HttpServlet.*/ 
     public ServletFrontController() {
		super();
	}

     
     
    // ***********************************************************************
    // ****    INICI DELS MÈTODES DE INICIALITZACIÓ DEL SAPPFRAMEWORK.    ****
    // ***********************************************************************          
     
     
     
     
     /**
     * <li> Mètode que inicialitza els components de l'aplicació.</li>
     * </li> S'executa una vegada durant la creació del servlet.</li>
     * @param conf parèmtres de configuració de l'aplicació, es prenen de l'arxiu web.xml.
      * @throws ServletException Errors d'inicialització del FrontEnd.
      * @throws ReflectionsException Conrtol d'errros de  Google Reglections 
     */
    @Override
    public void init(ServletConfig conf) throws ServletException, ReflectionsException {
        super.init(conf);
       logger.info("Inicialització del Servlet"); 
       errors = false;
     
       
        // Obtenir ruta de l'aplicació.
        rutaApp = conf.getInitParameter(RUTA_APLICACIO);
        // Obtenir pagina d'error per defecte
        paginaError = conf.getInitParameter(PAGE_ERROR);
        paginaErrorPrivilegis =conf.getInitParameter(PAGE_ERROR_PRIVILEGIS);        
        try{
            //Creem una instancia de Reflections per poder manipular-la
            Reflections reflections = new Reflections(rutaApp, new TypeAnnotationsScanner());
       
           
            // Escanejem els paquets amb l'ajuda de Google Reflections
            // per trobar les clases que incorporen controladors 
            Set<Class<?>> controladors = reflections.getTypesAnnotatedWith(Controlador.class);
            for (Class<?> classeControlador : controladors) {
                    
                 String nomControlador="";
                 String nomAccio ="";
                 Ccontrolador nouControlador = new Ccontrolador();
                 try{
                    logger.info("Inici Analisi controlador" + classeControlador.getName()+" i els les acions asociades");   
                    // Analitzem si el controlador correspon a un controlador d'usuaris
                    ControlUsuari contrs = classeControlador.getAnnotation(ControlUsuari.class);
                    if (contrs !=null){
                        nouControlador.setIsControUsuari(true);
                    }
                    
                    // Obtenim les anotacions del controlador.
                    Controlador contr = classeControlador.getAnnotation(Controlador.class);
                
                    // Anlitzar el nom del controlador
                    if (contr.nom() == null || "".equals(contr.nom().trim())) {
                        nomControlador = (classeControlador.getSimpleName().toLowerCase());
                    } else {
                        nomControlador = contr.nom();
                    }
                    
                    //Guardem la localització del controlador
                    mapacontroladors.put(nomControlador, classeControlador.getName());
                    mapacontroladorsnomreal.put(classeControlador.getName(), nomControlador);
                     
                    // Definir valors dels Anotations pel controlador
                    nouControlador.setName(nomControlador);
                    nouControlador.setClasseControlador(classeControlador);
                    nouControlador.setTipusExcucio(contr.tipusexecucio());
                 }
                 catch (Exception e){
                    logger.warning("Error a l'analitzar el controlador");
                    errors = true;

                 }

                 // Anàlisi d'accions dins d'un controlador
                 logger.info("Analitzar els mètodes que pertanyen definits com accions");      
                 Method[] metodes = classeControlador.getMethods();
                 for (Method metodeControlador : metodes) {
                     avaluarParamAccions(metodeControlador, nouControlador, classeControlador);

               }
                 // Inserir el controlador a la llista de controladors del FrontEnd
                 // Abans controlem que no existeix, no poden haver dos amb el mateix nom
                 if(!llistaControladors.containsKey(nomControlador)){
                     llistaControladors.put(nomControlador, nouControlador);
                 }
                 else{
                     logger.severe("Error: El controlador amb el nom "+nomControlador+" ja existeix."); 
                 }
             logger.info("Fi a nalisi controlador" + classeControlador.getName()); 
             }
           
        }
        catch(ReflectionsException e){
            logger.severe("Error en la incialitzazió del servlet durant la cerca de controladors i accions");    
        }
        
       logger.info("Fi inicialització del Servlet"); 
    }
    /**
    * Avalua un mètode per esbrinar si correspon a una preexecucios
    * @param metodeAccio Mètode a avaluar
    * @param classeControlador Controlador a que correspon aquest mètode
    * @return Retorna un array amb els paràmetres de la preexecució
    */
    protected Cpreproces[] avaluarParampreexe (Method metodeAccio, Class<?> classeControlador  ){
        boolean prextrobada =false;
        Map<String,Cpreproces> preprotemp =(Map<String,Cpreproces>) new HashMap();
        Method[] metodespre = classeControlador.getMethods();
        for (Method metodespreexec : metodespre) {
            
            if (metodespreexec.getAnnotation(Preexecucio.class)!=null){
                Annotation[] dc = metodespreexec.getDeclaredAnnotations();                   
                    //Extreure els paràmetres
                    Preexecucio anotpre = metodespreexec.getAnnotation(Preexecucio.class);
                    //Extreure el nom de l'acció associada
                    String[] anot= anotpre.metodeacio();

                for (int p = 0; p < anot.length; p=p+2) {
                    prextrobada =false;
                    //Si el nom coindiceix es que aquesta preexecució 
                    //Correpon al mètode passat com a paràmetre
                    if (metodeAccio.getName().equals(anot[p])){
                        prextrobada =true;
                    }
                    else{
                        if (anot[p] == null || "".equals(anot[p].trim())){
                           logger.severe("Error en la incialitzazió del servlet: Els paràmetres del mètode de precondició "+metodespreexec.getName()+" no son correctes o l'acció no existeix");    
                        }
                    }
                    //Si hem trobat una preexecuió recuperar paràmetres
                    if (prextrobada){
                        //Analitzar les anotacions del mètode
                        //que es passen com a paràmetres de l'acció
                        Class[] paramsClasses = metodespreexec.getParameterTypes();
                        Cparam[] paramsmetode = new Cparam[paramsClasses.length];
                        Cpreproces noupre = new Cpreproces(); 
                        paramsmetode=anotParam(metodespreexec);
                    
                    
                        //Carreguem els paràmetres de la preexecució
                        noupre.setParametres(paramsmetode);
                        //Definim el mètode de la preexecució
                        noupre.setMetode(metodespreexec);
                        //Controlador de la preexecució
                        noupre.setCAcció(metodeAccio);
                        //Carregar el número d'ordre d'execució
                        try{
                            noupre.setNumOrdre(Integer.parseInt(anot[p+1]) );
                        }
                        catch(Exception e){
                            logger.severe("Error en la incialitzazió del servlet: Els paràmetres del mètode de precondició "+metodespreexec.getName()+" no son correctes el segon paràmetre ha de ser un nombre enter que indica el número d'ordre de l'execució");    
                        }
                        //Carregar la pàgina de retorn en cas d'error
                        if(anotpre.retornError() == null || "".equals(anotpre.retornError().trim())){
                            noupre.setRetornError(this.paginaError);                           
                        }
                        else{
                            noupre.setRetornError(anotpre.retornError());  
                        }
                        //Avaluar preexecució
                        // Si ja existeix el mètode no tornem a carregar
                        if (!preprotemp.containsKey(metodespreexec.getName())){
                            preprotemp.put(metodespreexec.getName(), noupre);
                        }
                    }
                }
            } 
       }
        // Carrega de preprocessament
        //Primer avaluem el nombre d'elements a l'array
        Cpreproces[] prepro= new Cpreproces[preprotemp.size()];
        //Si hi ha valors els ordenem
        if (preprotemp.size() !=0){
            Collection c = preprotemp.values();
            prepro =ordenapre(c);
        }

        return prepro;
    }    
        /**
    * <li> Endreça els mètode de preexecució, aquestos mètodes s'ha d'enumerar de forma consecutiva. </li>
    * <li> El número d'ordre està situat a la Anotation després de la defició del nom del mètode pel qual d'ha d'executar el preprocessat.</li>
    * <li> No poden haver salts entre números, ha de ser expecificament consecutiu. </li>
    * @param cpre Collecció de mètodes de postexecució
    * @return Retorna un array amb els mètodes de la postexecució
    */    
    protected Cpreproces[] ordenapre (Collection cpre){
        boolean error =false;
        
        Cpreproces[] prepro= new Cpreproces[cpre.size()];
        Iterator iter1 = cpre.iterator();
        while (iter1.hasNext() & !error) {
            Cpreproces eva1 =  (Cpreproces) iter1.next();
            //Carregar el valor al l'array a la posició que l'hi toca si no està ocupada
            if (!error){
                try{
                    if (prepro[eva1.getNumOrdre()-1]!=null){
                        logger.severe("Error: La ordenació dels mètodes de preporcessat, no es pot dur a terme, error al mètode "+eva1.getMetode().getName() + ". El nombre d'ordre  està repetit"); 
                        error = true;
                    }
                    else{
                      prepro[eva1.getNumOrdre()-1] = eva1;  
                    }
                }
                catch(Exception e){
                    logger.severe("Error: La ordenació dels mètodes de preporcessat no es pot dur a terme error al mètode "+eva1.getMetode().getName() + ". El nombre d'ordre ha de ser consecutiu"); 
                    error = true;
                }
            }
            // Si hi han errors carregar un null i delagar la carrega a la classe
            else if(error){
                prepro =null;
            }
        }    
        return prepro;
    }
    
    
    /**
    * Avalua un mètode per esbrinar si correspon a una postexecucios
    * @param metodeAccio Mètode a avaluar
    * @param classeControlador Controlador a que correspon aquest mètode
    * @return Retorna un array amb els paràmetres de la postexecució
    */
    protected Cposproces[] avaluarParampostexe (Method metodeAccio, Class<?> classeControlador  ){
        boolean posttrobada =false;
        Map<String,Cposproces> postprotemp =(Map<String,Cposproces>) new HashMap();
        Method[] metodespost = classeControlador.getMethods();
        for (Method metodespostexec : metodespost) {
            if (metodespostexec.getAnnotation(Postexecucio.class)!=null){
                Annotation[] dc = metodespostexec.getDeclaredAnnotations();                   
                    //Extreure els paràmetres
                    Postexecucio anotpost = metodespostexec.getAnnotation(Postexecucio.class);
                    //Extreure el nom de l'acció associada
                    String[] anot= anotpost.metodeacio();

                for (int p = 0; p < anot.length; p=p+2) {
                    posttrobada =false;
                    //Si el nom coindiceix es que aquesta preexecució 
                    //Correpon al mètode passat com a paràmetre
                    if (metodeAccio.getName().equals(anot[p])){
                        posttrobada =true;
                    }
                    else{
                        if (anot[p] == null || "".equals(anot[p].trim())){
                           logger.severe("Error en la incialitzazió del servlet: Els paràmetres del mètode de precondició "+metodespostexec.getName()+" no son correctes o l'acció no existeix");    
                        }
                    }
                    //Si hem trobat una preexecuió recuperar paràmetres
                    if (posttrobada){
                        //Analitzar les anotacions del mètode
                        //que es passen com a paràmetres de l'acció
                        Class[] paramsClasses = metodespostexec.getParameterTypes();
                        Cparam[] paramsmetode = new Cparam[paramsClasses.length];
                        Cposproces noupost = new Cposproces(); 
                        paramsmetode=anotParam(metodespostexec);
                    
                    
                        //Carreguem els paràmetres de la preexecució
                        noupost.setParametres(paramsmetode);
                        //Definim el mètode de la preexecució
                        noupost.setMetode(metodespostexec);
                        //Controlador de la preexecució
                        noupost.setCAcció(metodeAccio);
                        //Carregar el número d'ordre d'execució
                        try{
                            noupost.setNumOrdre(Integer.parseInt(anot[p+1]) );
                        }
                        catch(Exception e){
                            logger.severe("Error en la incialitzazió del servlet: Els paràmetres del mètode de precondició "+metodespostexec.getName()+" no son correctes el segon paràmetre ha de ser un nombre enter que indica el número d'ordre de l'execució");    
                        }
                        //Carregar la pàgina de retorn en cas d'error
                        if(anotpost.retornError() == null || "".equals(anotpost.retornError().trim())){
                            noupost.setRetornError(this.paginaError);                           
                        }
                        else{
                            noupost.setRetornError(anotpost.retornError());  
                        }
                        //Avaluar preexecució
                        // Si ja existeix el mètode no tornem a carregar
                        if (!postprotemp.containsKey(metodespostexec.getName())){
                            postprotemp.put(metodespostexec.getName(), noupost);
                        }
                    }
                }
            } 
       }
        // Carrega de postprocessament
        //Primer avaluem el nombre d'elements a l'array
        Cposproces[] postpro= new Cposproces[postprotemp.size()];
        //Si hi ha valors els ordenem
        if (postprotemp.size() !=0){
            Collection c = postprotemp.values();
            postpro =ordenapost(c);
        }
        return postpro;
    }
    /**
    * <li> Endreça els mètode de postexecució, aquestos mètodes s'ha d'enumerar de forma consecutiva. </li>
    * <li> El número d'ordre està situat a la Anotation després de la defició del nom del mètode pel qual d'ha d'executar el postporcessat. </li>
    * <li> No poden haver salts entre números, ha de ser expecificament consecutiu. </li>
    * @param cpost Collecció de mètodes de postexecució
    * @return Retorna un array amb els mètodes de la postexecució
    */    
    protected Cposproces[] ordenapost (Collection cpost){
        Boolean error =false;
        Cposproces[] postpro= new Cposproces[cpost.size()];
        Iterator iter1 = cpost.iterator();
        while (iter1.hasNext() & !error) {
            Cposproces eva1 =  (Cposproces) iter1.next();
            //Carregar el valor al l'array a la posició que l'hi toca si no està ocupada
            if (!error){
                try{
                    if (postpro[eva1.getNumOrdre()-1]!=null){
                        logger.severe("Error: La ordenació dels mètodes de postporcessat, no es pot dur a terme, error al mètode "+eva1.getMetode().getName() + ". El nombre d'ordre  està repetit"); 
                        error = true;
                    }
                    else{
                      postpro[eva1.getNumOrdre()-1] = eva1;  
                    }
                }
                catch(Exception e){
                    logger.severe("Error: La ordenació dels mètodes de postporcessat no es pot dur a terme error al mètode "+eva1.getMetode().getName() + ". El nombre d'ordre ha de ser consecutiu"); 
                    error = true;
                }
            }
            // Si hi han errors carregar un null i delagar la carrega a la classe
            else if(error){
                postpro =null;
            }
        }    
        return postpro;
    }
    
    /**
    * Avalua un mètode d'un controlador i els insereix les accions del controlador amb els seus paràmetres
    * @param metodeControlador Mètode a avaluar
    * @param nouControlador Controlador a que correspon aquest mètode
    * @param classeControlador Classe del controlador
    * @return  Retorna false si hi ha algún error durant l'anànalisi dels paràmetres
    */
    protected boolean avaluarParamAccions (Method metodeControlador, Ccontrolador nouControlador, Class<?> classeControlador  ){
        
        String nomControlador= nouControlador.getNom();
        String nomAccio ="";
        boolean error = false;
        Caccio novaaccio = new Caccio(); 
        Accio accio = metodeControlador.getAnnotation(Accio.class);

        //Determinar si el mètode es una acció
        if (accio !=null){

            // Si no hi ha nom definit, utilitzarem el nom de la classe.
            if (accio.nom() == null || "".equals(accio.nom().trim())) {
                nomAccio = metodeControlador.getName();
                novaaccio.setNom(nomAccio);
            } else {
                nomAccio =accio.nom();
                novaaccio.setNom(nomAccio);
            }
              // Analitzem si hi pàgina de retorn definida
              // Si no n'hi ha retornarà generem un avís
              if (accio.retorn() != null && !"".equals(accio.retorn().trim())) {
                 novaaccio.setRetorn(accio.retorn());
              }
              else{
                  logger.severe("Error: L'acció amb el nom "+novaaccio.getNom()+" no te la pàgina de retorn definida.");  
              }
              // Analitzem si hi pàgina de retorn definida en cas d'error
              // Si no n'hi ha retornarà a la pàgina per defecte del servlet
              if (accio.retornError() != null && !"".equals(accio.retornError().trim())) {
                novaaccio.setRetornError(accio.retornError());
              }
              else{
                novaaccio.setRetornError(paginaError); 
              }
              //Analitzar les anotacions del mètode
              //que es passen com a paràmetres de l'acció
              Class[] paramsClasses = metodeControlador.getParameterTypes();
              Cparam[] paramsmetode = new Cparam[paramsClasses.length];
              paramsmetode=anotParam(metodeControlador);

              //Carreguem els paràmetres de l'acció
              novaaccio.setParametres(paramsmetode);
              //Definim el mètode de l'acció
              novaaccio.setMetode(metodeControlador);
              //Controlador de l'acció
              novaaccio.setControlador(nouControlador);
              
              //Avaluar rols
              Rol prol = metodeControlador.getAnnotation(Rol.class);
              if (prol !=null){
                  novaaccio.setRols(prol.rols());
              }
              
              //Avaluar mètodes de preexecució
              Cpreproces[] prepro = avaluarParampreexe(metodeControlador, classeControlador );
              if (prepro!=null){
                 novaaccio.setPreexec(prepro);   
              }
              
              //Avaluar mètodes de postexecució
              Cposproces[] pospro = avaluarParampostexe(metodeControlador, classeControlador );
              if (pospro!=null){
                 novaaccio.setPostexec(pospro);   
              }
                      
              //Analitzem si l'acció ja existeix al controlador
              //Si no existeix la inserim
              if (!nouControlador.getAccions().containsKey(novaaccio.getNom())){
                nouControlador.setAccio(novaaccio.getNom(), novaaccio);
                mapaaccions.put(nomControlador+"."+nomAccio, metodeControlador.getName());
                error = true;
              }
              else{
                logger.severe("Error: L'acció amb el nom "+novaaccio.getNom()+" ja existeix."); 
             }
           }
    return error;
    }
      /**
     * Analitzar les anotacions del mètode que es passen com a paràmetres 
     * @param metode Mètode a analitzar.
     * @return Retorna un array amb els paràmetres de l'acció.
     */
    protected Cparam[] anotParam (Method metode){
        
              //Analitzar les anotacions del mètode
              //que es passen com a paràmetres 
              Annotation[][] paramsAnot = metode.getParameterAnnotations();
              Class[] paramsClasses = metode.getParameterTypes();
              Cparam[] paramsmetode = new Cparam[paramsClasses.length];
              Type[] paramsGenericTypes = metode.getGenericParameterTypes();
              Cparam paramEntrada = null;
              for (int i = 0; i < paramsAnot.length; i++) {
                // Analitzar si exiteix definidió del paràmetre.
                if (paramsAnot[i].length < 1) {
                    logger.warning("Error: falta definició del paràmetre");
                }
                paramEntrada = null;
                for (int j = 0; j < paramsAnot[i].length; j++) {
                  // Analitzem els parèmtres que provenen de l'aplicació
                  if (ParamApp.class.equals(paramsAnot[i][j].annotationType())) {
                    //parèmtres que provenen de l'aplicació
                    ParamApp paramapp = (ParamApp) paramsAnot[i][j];
                    paramEntrada = new Cparam();
                    paramEntrada.setAmbit("aplicacio");
                    paramEntrada.setNomClasse(paramapp.nom());
                    paramEntrada.setClasse(paramsClasses[i]);
                    paramEntrada.setParamGenericType(paramsGenericTypes[i]);
                  }    
                  else if (ParamWeb.class.equals(paramsAnot[i][j].annotationType())) {
                    //Paràmetres que provenen de la web
                    ParamWeb paraweb = (ParamWeb) paramsAnot[i][j];
                    paramEntrada = new Cparam();
                    paramEntrada.setAmbit("web");
                    paramEntrada.setNom(paraweb.nom());
                    paramEntrada.setClasse(paramsClasses[i]);
                    paramEntrada.setParamGenericType(paramsGenericTypes[i]);
                  }    
                  else if (ParamRequest.class.equals(paramsAnot[i][j].annotationType())) {
                    //El paràmetre es la request
                    ParamRequest paraRec = (ParamRequest) paramsAnot[i][j];
                    paramEntrada = new Cparam();
                    paramEntrada.setAmbit("request");
                  }    
                  else if (!ParamApp.class.equals(paramsAnot[i][j].annotationType()) &
                    !ParamWeb.class.equals(paramsAnot[i][j].annotationType()) ) {
                    // El paràmetre no correspon a cap de definit.
                    logger.warning("Error: a l'analitzar els paràmetre del mètode "+ metode.getName()+" Tipus paràmetre desconegut");;
                  }
                }
                if (paramEntrada == null) {
                  logger.warning("Error: No s'ha trobat cap parèmere");
                }
                paramsmetode[i] = paramEntrada;
             }
    return paramsmetode;          
    }
    // ***********************************************************************
    // ****       FI DELS MÈTODES DE INICIALITZACIÓ DEL SAPPFRAMEWORK.    ****
    // ***********************************************************************     
    
    
    // *********************************************************************************
    // ****       INICI DELS MÈTODES DE CONTROLS DE PETICIONS DEL SAPPFRAMEWORK.    ****
    // *********************************************************************************         
    
    
    
    /**
     * <li> Aquest mètode s'executa en rebre una petició des de la web. Es l'encarregat de gestionar totes les peticions.</li>
     * <li> Retornarà la resposta cap a la pàgina que correspongui i sempre inserirá l'atribut "Sapp" que és el que conté tota la informació dels controladors en curs. </li>
     * @param request Request de la peició actual
     * @param response Reposta de la petició
     * @throws ServletException Control d'erros del servlet
     * @throws IOException Control d'error d'entrdada sortida
     */
    @Override    
    protected void service(HttpServletRequest request, HttpServletResponse response)
                   throws ServletException, IOException {
	String nextPage=null;
        
        //Executar acció
        nextPage = this.executeAction(request, response);
        
        //Comprova que la pàgina es plena si no és així carreguem l´última
        if (nextPage == null || "".equals(nextPage.trim())){
             Object o = this.getServletContext().getAttribute("ultimapagina");
             nextPage = (String) o;
        }
        ultimapagina = nextPage;
        this.getServletContext().setAttribute("ultimapagina", ultimapagina);
        
            //executar codi accions
            nextPage=("/"+nextPage+".jsp"); 
            try {
                RequestDispatcher rd = getServletContext().getRequestDispatcher(response.encodeURL(nextPage));
                
                rd.forward(request, response);
            } catch (Throwable t) {
                t.printStackTrace();
            }
    }

    /**
     * Analitza l'ambit de l'execució d'una acció.
     * @param request Petició actual.
     * @param controlador controlador asociat a la petició
     * @return Retorna un objecte que correspon a la classe de l'acció
     * @throws ServletException Control d'errors del servlet 
     */
    protected Object analisiAmbit(HttpServletRequest request, Ccontrolador controlador)
		throws ServletException {
        Object o=null;
        try{
            HttpSession session = request.getSession();
            if (controlador.getTipusExcucio().equals(Constants.TIPUS_EXECUCIO_SESSIO)){
                // Examinar si la sessió ja està creada
                
                if (mapaSessions.containsKey(session.getId())){
                    //Si la sessió existeix recuperem el mapa de classes de la sessió
                    //Map <String, Object> ctrlsessio = new HashMap (mapaSessions.get(session.getId()));
                    Map <String, Object> ctrlsessio = mapaSessions.get(session.getId());
                    //Recuperem la instancia de la classe de l'acció que s'ha de dur a terme
                    o = ctrlsessio.get(controlador.getNom());
                    // Si l'objecte no existeix l'inserim
                    if (o==null){
                        try{
                          o = controlador.getClasseControlador().newInstance();
                        } catch(Exception e){
                            logger.warning("Error: No s'ha pogut instanciar la classe "+ controlador.getNom() +" durant l'anàlisi de l'àmbit" );   
                        }
                        ctrlsessio.put(controlador.getNom(), o);
                    }
                }
                else{
                    //Si la sessió no existeix la creem, creem el mapa de controlador i inserim la classe que correspongui
                    HashMap <String, Object> ctrlsessio = new HashMap ();
                    try{
                        o = controlador.getClasseControlador().newInstance();
                    }
                    catch(Exception e){
                        logger.warning("Error: No s'ha pogut instanciar la classe "+ controlador.getNom() +" durant l'anàlisi de l'àmbit" );   
                    }
                ctrlsessio.put(controlador.getNom(), o);
                mapaSessions.put(session.getId(), ctrlsessio); 
                }
            }      
            if (controlador.getTipusExcucio().equals(Constants.TIPUS_EXECUCIO_PR)){
                
                // Crear el objecte controlador només per la petició en cur
                // No s'inclou a dis del mapa de controladors de sessió
                o = controlador.getClasseControlador().newInstance();
            }
        }
        catch(Exception e){
            logger.warning("Error: Durant l'anàlisi de l'ambit de l'acció " + controlador.getNom() ); 
        }
       
        
        return o;
    }
    
    
     /**
     * Adatador de dades de sortida per implementar un framework com per exemple Freemarker.
     * Aquest mètode només s'ha d'implmentar en el cas de voles convertir
     * les dades de sortida de SappFramework a un altre framework
     * @param mapacontroladors Mapa de controladors de entrada procedents de SappFramework.
     * @return Retorna mapa confecionat en funció del framework que ev vulgui implementar
     */
    protected Map<String, Object> adaptador (Map mapacontroladors ) {
        Map<String, Object> ret = mapacontroladors;
        return ret;
    }
    
      /**
     * Recupera els paràmetres de la web per carregarlos a l'acció.
     * @param request Petició actual.
     * @param paract Paràmeres de l'acció actual
     * @param obj Acció a la que correspon la petició actual
     * @param error Control dels errors
     * @return Retorna un objecte que correspon a la classe de l'acció
     * @throws ServletException Control d'errors del servlet 
     */
    protected Object[] parametresRes(HttpServletRequest request,Cparam[] paract, Object obj, boolean error)
                    throws ServletException {
        Map<Class, IConversor> convertidor = this.getConversosrs();
        error=false;
        Enumeration<String> paramNames = request.getParameterNames();
        Object[] paramsortida = new Object[paract.length];
        int i =0;
        for (Cparam actparact : paract) {
            
            //Controlem que el paràmetre sigui del tipus web
            if (actparact.getAmbit().equals("web")){
                //Extraiem el conversor
                IConversor conver = convertidor.get(actparact.getClasse());
                //Extraiem el valor   
                String[] valor = request.getParameterValues(actparact.getNom());
                //Convertim el valor
                Object o = conver.conversio(valor);
                //Preparem nom del mètode get i set, la primera ha d'anar amb majuscules
                String nom = actparact.getNom();
                String nomup="";        
                if(nom == null) {
                    nomup="";
                    error=true;
                } else if(nom.length() == 1) {
                    nomup = nom.toUpperCase();
                } else {
                    nomup= nom.substring(0,1).toUpperCase() + nom.substring(1);
                }
                // Recuperar mètodes set i get per actualitzar dades.
                try{    
                    Method getter = obj.getClass().getMethod("get" + nomup);
                    Method setter = obj.getClass().getMethod("set" + nomup, getter.getReturnType());
                    //Invoquem el mètode set per carregar el valor
                    setter.invoke(obj, o);
                    paramsortida[i] = getter.invoke(obj); 
                }
                catch(Exception e){
                    logger.warning("Error: Mètode set"+nomup+ " o get"+nomup+ " de la classe  "+obj.getClass().getSimpleName()+" no trobat. Comprovar tipus dels mètodes"  );
                    error =true;
                }
            }
            //Analisis de paràmetres de l'aplicació
            else if (actparact.getAmbit().equals("aplicacio")){
                //Buscar si el paràmetre es una classe controlador
                if (mapacontroladorsnomreal.containsKey(actparact.getClasse().getName())){
                    Ccontrolador contc = llistaControladors.get(mapacontroladorsnomreal.get(actparact.getClasse().getName()));
                    
                    //S'ha de controlar si l'ambit es d'aplicació o de sessió per instanciar la classe
                    // que es passa com a paràmetre i guardar-la a nivell de petició o de sessió
                    
                    
                    
                    try{
                        
                        //controlar l'ambit de l'apliació per generar l'objecte del paràmetre
                        paramsortida[i]=analisiAmbit(request, contc);
                        //paramsortida[i] = contc.getClasseControlador().newInstance(); 
                    }
                    catch(Exception e){
                        logger.warning("Error: No s'hapogut instaciar la classe "+ contc.getClasseControlador().getName()+" durant el procés d'adquisició de dades");
                        error = true;    
                    }
                }
            }
              else if (actparact.getAmbit().equals("request")){
                  // si el paràmetre actual te el ambit de la request carreguem la request actual
                  paramsortida[i] = request;
              }
            //Incremetar punter sortida paràmetres
            i++;
        }    
        return paramsortida;
    }
     /**
     * Retorna el mapa de conversors.
     * @return Conversors .
     */
    private Map<Class, IConversor> getConversosrs() {
        Map<Class, IConversor> defaultConverters = new HashMap<Class, IConversor>();
        defaultConverters.put(Boolean.class, new ConverClasseBoolean());
        defaultConverters.put(Double.class, new ConverClasseDouble());
        defaultConverters.put(Float.class, new ConverClasseFloat());
        defaultConverters.put(Integer.class, new ConverClasseInteger());
        defaultConverters.put(Long.class, new ConverClasseLong());
        defaultConverters.put(String.class, new ConverString());
        defaultConverters.put(boolean.class, new Converbolean());
        defaultConverters.put(double.class, new Converdouble());
        defaultConverters.put(float.class, new Converfloat());
        defaultConverters.put(int.class, new Converint());
        defaultConverters.put(long.class, new Converlong());
        return defaultConverters;
    }
     /**
     * Executa l'acció rebuda per la request.
     * @param request Request de la petició actual 
     * @param response response de la petició actual
     * @return String amb el valor de la pàgina de sortida .
     * @throws ServletException Control d'error del servlet
     * @throws IOException Control d'errors d'entrada sortida 
     */    
    protected String executeAction(HttpServletRequest request, HttpServletResponse response)
		throws ServletException,IOException {
        errors = false;
        String nextpage = paginaError;
        HttpSession session = 	request.getSession();
        session.getId();

        //Analitzar el QueryString per cercar el controlador i l'acció
        Ccontrolador classeContr=null;
        Caccio metodeActi=null;
        String contr="";
        String acti="";
        try{
            String qs = request.getQueryString();
            contr = qs.substring(qs.lastIndexOf("/")+1,qs.lastIndexOf("."));
            acti = qs.substring(qs.lastIndexOf(".")+1);
            
            //Si hi ha un interrogant es que hi ha paràmetres passats per URL
            if (acti.contains("?")){
                String acti2 = acti;
                acti = acti.substring(0, acti.lastIndexOf("?"));
            }
            
        }
        catch(Exception e){
             logger.warning("Error: La QueryString no està ben formada");
             request.setAttribute("error", "Error: La QueryString ("+ request.getQueryString() +") no està ben formada ");
             errors = true;
        }
        if (!errors){
            if (llistaControladors.containsKey(contr)& !errors){
                classeContr=llistaControladors.get(contr);

                if (classeContr.getAccions().containsKey(acti) & !errors){
                metodeActi=classeContr.getAccions().get(acti);
                }
                else{
                    logger.warning("Error: no s'ha trobat l'acció "+acti);
                    request.setAttribute("error", "Error: no s'ha trobat l'acció " + acti);
                    errors = true;
                }
            }
            else{
                logger.warning("Error: no s'ha trobat el controldor "+contr);
                request.setAttribute("error", "Error: no s'ha trobat el controldor "+contr);
                errors = true;
            }
        }

 
        //Analitzar l'ambit de la petició
        // retorna la instancia de l'objecte de la classe a que correspon l'acció que s'ha de dur a terme
        Object clsseins =null;
        if (!errors){
            clsseins =  analisiAmbit(request, classeContr);
            if (clsseins==null){
           // Si no optenim cap instancia es que hi ha hagut un error durant l'anàlisi de l'àmbit
            errors =true;
            }
        }
 
        // Recuperar i executar mètodes de preprocessat si n'hi han
        if (!errors){
            Cpreproces[] prepro =(Cpreproces[]) metodeActi.getPreexec();
            if (prepro!=null){
                 for (int i = 0; i < prepro.length; i++) {
                    try{    
                        Object[] paramspre = null;
                        paramspre = parametresRes(request, prepro[i].getParametres(), clsseins, errors);
                        nextpage = prepro[i].getRetornError();
                        Method pre = clsseins.getClass().getMethod(prepro[i].getMetode().getName(), prepro[i].getMetode().getParameterTypes());
                        Object o = pre.invoke(clsseins, paramspre);
                        errors =(Boolean) o;
                        if (errors){
                            try{
                                Method met2 = clsseins.getClass().getMethod("getError", (Class<?>[])new Class<?>[0]);
                                Object o2 =met2.invoke(clsseins, (Object[])new Object[0]);
                                request.setAttribute("error", (String) o2);
                            } catch(NoSuchMethodException e){
                                logger.severe("Error no definit. No s'ha trobat el mètode getError a la classe "+ clsseins.getClass().getName());
                                request.setAttribute("error", "Error no definit. No s'ha trobat el mètode getError a la classe "+ clsseins.getClass().getName());
                            }
                             break;
                        }
                    } catch(InvocationTargetException e){
                        logger.severe("Error: no s'ha executat el mètode "+ metodeActi.getNom()+" dela classe "+metodeActi.getControlador().getNom());
                        Throwable cause = e.getCause();
                        e.printStackTrace();
                        errors =true;
                    } catch (IllegalAccessException e) {
                        logger.severe("Error: no s'ha executat el mètode "+ metodeActi.getNom()+" dela classe "+metodeActi.getControlador().getNom());
                        Throwable cause = e.getCause();
                        e.printStackTrace();
                        errors =true;
                    } catch (NoSuchMethodException e) {
                        logger.severe("Error: no s'ha executat el mètode "+ metodeActi.getNom()+" dela classe "+metodeActi.getControlador().getNom());
                        Throwable cause = e.getCause();
                        e.printStackTrace();
                        errors =true;
                    }
                 }
            }
        }
 
        // Recuperar els paràmetres de l'acció
        Object[] paramsapp =null;
        if (!errors){
            paramsapp =parametresRes(request, metodeActi.getParametres(), clsseins, errors);
            if(paramsapp.length != metodeActi.getParametres().length ){
               errors=true;
            }
        }
        
        //Execució de l'acció
        if (!errors){
            boolean errorrol =false;
            try{
                //Execució de l'acció
                Method met = clsseins.getClass().getMethod(metodeActi.getMetode().getName(), metodeActi.getMetode().getParameterTypes());
                nextpage = metodeActi.getRetornError();
                // Si la petició es al controlador d'usuaris
                if (classeContr.getIsControUsuari() && met.getName().equals("validarSessio")){
                    //Avaluar si està al procés de validació
                    //si es així el returntype serà diferen
                    if (met.getName().equals("validarSessio")){
                        Object o =met.invoke(clsseins, paramsapp);
                        IUsuari usuari = (IUsuari) o; 
                        mapaSessions.get(request.getSession().getId()).put("SappUsuariContext", usuari);

                        if (!usuari.getValidat()){
                            errors=true;
                        }
                    }
                }
                //Si es una acció normal executem normalemtn
                else{
                //Control de rols
                    boolean rolok = false;
                    if (metodeActi.getRols() != null ){
                        for (int i = 0; i < metodeActi.getRols().length; i++){
                            IUsuari usuari = (IUsuari) mapaSessions.get(request.getSession().getId()).get("SappUsuariContext");
                            if (usuari!=null){
                            String[] rol = metodeActi.getRols();
                            if (rol[i].equals(usuari.getRol())){
                                Object o =met.invoke(clsseins, paramsapp);
                                errors =(Boolean) o;
                                rolok =true;
                                break;
                            }
                            }else {
                                
                            }
                        }
                        if (!rolok){
                            //Si estem aquí es que l'usuari no te prous privilegis per executra l'acció
                            errorrol=true;
                            errors=true;
                        }
                        
                    }else{ // Si no hi ha rols executem sense analitzar el rol de l'usuari
                            Object o =met.invoke(clsseins, paramsapp);
                            errors =(Boolean) o;
                    }
                }
                //Control dels errors
                //Si hi ha un error codificat al controlador el col·locarà al atribut error
                if (!errors){
                    nextpage = metodeActi.getRetorn();
                } else{
                    if (!errorrol){
                        try{
                        Method met2 = clsseins.getClass().getMethod("getError", (Class<?>[])new Class<?>[0]);
                        Object o2 =met2.invoke(clsseins, (Object[])new Object[0]);
                        request.setAttribute("error", (String) o2);
                        } catch(NoSuchMethodException e){
                             logger.severe("Error no definit. No s'ha trobat el mètode getError a la classe "+ clsseins.getClass().getName());
                             request.setAttribute("error", "Error no definit. No s'ha trobat el mètode getError a la classe "+ clsseins.getClass().getName());
                        }
                    }else{
                        //Si hi hau un error de rol carrega pagina de error de privilegis
                        nextpage =paginaErrorPrivilegis;
                    }
                }
            } catch(InvocationTargetException e){
                logger.severe("Error: no s'ha executat el mètode "+ metodeActi.getNom()+" dela classe "+metodeActi.getControlador().getNom());
                Throwable cause = e.getCause();
                e.printStackTrace();
                errors=true;
            } catch (IllegalAccessException e) {
                logger.severe("Error: no s'ha executat el mètode "+ metodeActi.getNom()+" dela classe "+metodeActi.getControlador().getNom());
                Throwable cause = e.getCause();
                e.printStackTrace();
                errors=true;
            } catch (NoSuchMethodException e) {
                logger.severe("Error: no s'ha executat el mètode "+ metodeActi.getNom()+" dela classe "+metodeActi.getControlador().getNom());
                Throwable cause = e.getCause();
                e.printStackTrace();
                errors=true;
            } 
        }

        // Si no hi han agut errors Recuperar i executar mètodes de postpreocessat si n'hi han
        if (!errors){
            Cposproces[] pospro =(Cposproces[]) metodeActi.getPostexec();
            if (pospro!=null){
                for (int i = 0; i < pospro.length; i++) {
                    try{    
                        Object[] paramspost = null;
                        paramspost = parametresRes(request, pospro[i].getParametres(), clsseins, errors);
                        Method pre = clsseins.getClass().getMethod(pospro[i].getMetode().getName(), pospro[i].getMetode().getParameterTypes());
                        Object o = pre.invoke(clsseins, paramspost);
                        errors =(Boolean) o;
                        if (errors){
                            nextpage = pospro[i].getRetornError();
                            try{
                                Method met2 = clsseins.getClass().getMethod("getError", (Class<?>[])new Class<?>[0]);
                                Object o2 =met2.invoke(clsseins, (Object[])new Object[0]);
                                request.setAttribute("error", (String) o2);
                            } catch(NoSuchMethodException e){
                                logger.severe("Error no definit. No s'ha trobat el mètode getError a la classe "+ clsseins.getClass().getName());
                                request.setAttribute("error", "Error no definit. No s'ha trobat el mètode getError a la classe "+ clsseins.getClass().getName());
                            }
                             break;
                        }
                    } catch(InvocationTargetException e){
                        logger.severe("Error: no s'ha executat el mètode "+ metodeActi.getNom()+" dela classe "+metodeActi.getControlador().getNom());
                        Throwable cause = e.getCause();
                        e.printStackTrace();
                        errors=true;
                    } catch (IllegalAccessException e) {
                        logger.severe("Error: no s'ha executat el mètode "+ metodeActi.getNom()+" dela classe "+metodeActi.getControlador().getNom());
                        Throwable cause = e.getCause();
                        e.printStackTrace();
                        errors=true;                    
                    } catch (NoSuchMethodException e) {
                        logger.severe("Error: no s'ha executat el mètode "+ metodeActi.getNom()+" dela classe "+metodeActi.getControlador().getNom());
                        Throwable cause = e.getCause();
                        e.printStackTrace();
                        errors=true;                    
                    }
                }
            }
        }
        //Controlar l'ambit d'execució per esbrinar si s'ha de
        //de passar l'objecte a la sortida de dades de la sessió
        //Si és ambit de sessió no cal guardar ja que s'ha guardat durant l'anàlisi de l'àmbit
        //Si l'ambit es petició resposta, inserim l'objecte a la sortida de dades
        sortidadesdesessio = new HashMap <String, Object> ();

        
        //Si el tipus d'execusió és petició resposta guardem el resultat de la petició
        //al mapa de sortida.
        if (classeContr.getTipusExcucio().equals(Constants.TIPUS_EXECUCIO_PR)){
            sortidadesdesessio.put(classeContr.getNom(), clsseins);
        }
        sortidadesdesessio.putAll(mapaSessions.get(request.getSession().getId()));
        //Ejecutra l'adaptador per altres frameworks com per exemple freemarker.
        sortidadesdesessio =this.adaptador(sortidadesdesessio);
        request.setAttribute("Sappf", sortidadesdesessio);
        
        return nextpage;
    }
}
