package SappFramework.Gestio;

import java.lang.reflect.Method;
import java.util.Map;

/**
 * Modela els atributs i elements que composen una l'acció del framework.
 * @author Josep Jiménez
 */
public class Caccio {

    private String nom;
    private String retorn;
    private String retornError;
    private Method metode;
    private Ccontrolador controlador;
    private Cpreproces[] preexecucio;
    private Cposproces[] postexecucio;
    private String[] rols;
    private Cparam[] params;

    /**
     * Retorna la redefinició del nom de l'acció del controlador.
     * @return El nom de l'acció.
     */
    public String getNom() {
        return nom;
    }

    /**
     * Actualització del nom de l'acció.
     * @param nom Nom de l'acció.
     */
    public void setNom(String nom) {
        this.nom = nom;
    }

    /**
     * Retorna el mètode de l'acció.
     * @return Mètode de l'acció.
     */
    public Method getMetode() {
        return metode;
    }

    /**
     * Defineix el mètode de l'acció.
     * @param metode Mètode de l'acció.
     */
    public void setMetode(Method metode) {
        this.metode = metode;
    }

    /**
     * Retorna la pàgina de retorn de l'acció.
     * @return Pàgina de retorn de l'acció.
     */
    public String getRetorn() {
        return retorn;
    }

    /**
     * Defineix la pàgina de retorn de l'acció.
     * @param retorn Pàgina de retorn de l'acció.
     */
    public void setRetorn(String retorn) {
        this.retorn = retorn;
    }

        /**
     * Retorna la pàgina de retorn si hi ha  error a l'acció.
     * @return Pàgina de retorn de l'acció en cas d'error.
     */
    public String getRetornError() {
        return retornError;
    }

    /**
     * Defineix la pàgina de retorn d'error de l'acció.
     * @param retornerror Pàgina de retorn en cas d'error de l'acció.
     */
    public void setRetornError(String retornerror) {
        this.retornError= retornerror;
    }
    
    
    /**
     * Retorna el controlador on es troba l'acció.
     * @return Controlador on es troba definida l'acció.
     */
    public Ccontrolador getControlador() {
        return controlador;
    }

    /**
     * Determina el controlador on es troba l'acció.
     * @param controlador Controlador de l'acció.
     */
    public void setControlador(Ccontrolador controlador) {
        this.controlador = controlador;
    }

    /**
     * Retorna els mètode de preexecució.
     * @return Mètodes de preexecució de de l'acció.
     */
    public Cpreproces[] getPreexec() {
        return preexecucio;
    }

    /**
     * Defineix la taula de mètodes de preexecució.
     * @param preexecucio Mètodes de prexecució de l'acció.
     */
    public void setPreexec(Cpreproces[] preexecucio) {
        this.preexecucio = preexecucio;
    }

    /**
     * Retorna els mètode de la postexecució.
     * @return Mètodes de la postexecució de de l'acció.
     */
    public Cposproces[] getPostexec() {
        return postexecucio;
    }

    /**
     * Defineix la taula de mètodes de la postexecució.
     * @param postexecucio Mètodes de la postexecució de l'acció.
     */
    public void setPostexec(Cposproces[] postexecucio) {
        this.postexecucio = postexecucio;
    }
    
    /**
     * Retorna un array amb els paràmetres d'entrada de l'acció.
     * @return Paràmetres de l'acció.
     */
    public Cparam[] getParametres() {
        return params;
    }

    /**
     * Defineix l'array dels paràmetres d'entrada de l'acció.
     * @param params Paràmetres de l'acció.
     */
    public void setParametres(Cparam[] params) {
        this.params = params;
    }
    
    /**
     * Retorna un array amb els rols de l'acció.
     * @return Rols de l'acció.
     */
    public String[] getRols() {
        return rols;
    }

    /**
     * Defineix l'array dels rols de l'acció si en te.
     * @param rols Rols de l'acció.
     */    
    public void setRols(String[] rols) {
        this.rols = rols;
    }

}
