package SappFramework.Gestio;

import java.util.Map;
import java.util.HashMap;


/**
 * Modela els elements i atributs que composen un controlador.
 * @author Josep Jiménez
 */
public class Ccontrolador {

    private String nom;
    private String tipusExecucio;
    private Map<String, Caccio> accions = new HashMap<String, Caccio>();
    private Class classeControlador;
    private boolean iscontrolusuari;


    /**
     * Retorna la redefinició del nom del controlador.
     * @return Nom del controlador.
     */
    public String getNom() {
        return nom;
    }

    /**
     * Actualització del nom del controlador.
     * @param nom Nom del controlador.
     */
    public void setName(String nom) {
        this.nom = nom;
    }

    /**
     * Retorna el tipus d'excució del controlador (sessió o petició/resposta).
     * @return L'àmbit.
     */
    public String getTipusExcucio() {
        return tipusExecucio;
    }

    /**
     * Determina el tipus d'execució del controlador.
     * @param tipusExecucio pot ser de sessió o de petició/resposta.
     */
    public void setTipusExcucio(String tipusExecucio) {
        this.tipusExecucio = tipusExecucio;
    }

    /**
     * Retorna les accions definides al controlador.
     * @return Les accions instal·lades.
     */
    public Map<String, Caccio> getAccions() {
        return accions;
    }

    /**
     * Enmagatzema les accions definides al controlador.
     * @param accio Accions del controlador.
     */
    public void setAccio(String nom, Caccio accio) {
        this.accions.put(nom, accio);
    }

    /**
     * Retorna la classe que implementa el controlador.
     * @return Classe del controlador.
     */
    public Class getClasseControlador() {
        return classeControlador;
    }

    /**
     * Defineix la classe que implementa el controlador.
     * @param controlador Classe del controlador.
     */
    public void setClasseControlador(Class controlador) {
        this.classeControlador = controlador;
    }

    /**
     * Retorna true si el controlador es un contralador d'usuari.
     * @return Un bolea que indica que .
     */
    public boolean getIsControUsuari() {
        return this.iscontrolusuari;
    }

    /**
     * Indica la propiestat de controlador d'usuari.
     * @param iscontrousuari Indica que controlador es un contrlador d'usuari.
     */
    public void setIsControUsuari(boolean iscontrousuari) {
        this.iscontrolusuari = iscontrousuari;
    }

 
}
