package SappFramework.Gestio;

import java.lang.reflect.Type;

/**
 * Modela els parèmtres i atributs d'un paràmetre d'entrada d'una acció.
 * @author Josep Jiménez
 */
public class Cparam {

    /**
    * <li>Àmbit del paràmtre.</li>
    * <li>L'ambit del parèmtre pot ser de tres tipus.</li>
    * <li>Dins l'aplicació, correspon amb Annotation ParamApp.</li>
    * <li>Dins la request, correspon amb Annotation ParamWeb.</li>
    * <li>La mateixa request ParamRequest.</li>
    */ 
    private String ambit;
     /**
    * <li>Nom del paràmetre.</li> 
    * <li>El nom ha de correspondrea un paràmetre del mètode i ha de tenir definits els mètodes accessors get i set.</li>
    */
    private String nom;
    
     /** 
     * Nom de la classe d'un paràmetre de nivell d'aplicació
     */
    private String  nomclasse;
    /** 
     * Classe a la que pertany el paràmetre
     */
    private Class  classe;
    /**
     * Tipus del paràmetre
     */
    private Type   tipusgeneric;

    /**
     * Retorna el l'ambit en que s'ha de buscar el valor.
     * @return Ambit on es troba el valor.
     */
    public String getAmbit() {
        return ambit;
    }

    /**
     * Defineix l'ambit on es buscarà el valor del paràmetre.
     * @param ambit del parèmtre.
     */
    public void setAmbit(String ambit) {
        this.ambit = ambit;
    }

    /**
     * Retorna el nom del paràmetre de l'acció.
     * @return Nom del paràmetre.
     */
    public String getNom() {
        return nom;
    }

    /**
     * Defineix el nom del paràmetre.
     * @param nom Nom del paràmetre
     */
    public void setNom(String nom) {
        this.nom = nom;
    }

    /**
     * Retorna el nom de la classe d'un paràmetre que correspon a l'àmbit d'aplicació.
     * @return Nom de la classe.
     */
    public String getNomClasse() {
        return nomclasse;
    }

    /**
     * Determina el nom de la classe d'un paràmetre que correspon a l'àmbit d'aplicació.
     * @param nomclasse Nom de la classe.
     */
    public void setNomClasse(String nomclasse) {
        this.nomclasse = nomclasse;
    }

    /**
     * Retorna el classe Java del paràmetre de l'acció.
     * @return Classe java.
     */
    public Class getClasse() {
        return classe;
    }

    /**
     * Defineix la classe java del paraàmetre.
     * @param classe Classe java.
     */
    public void setClasse(Class classe) {
        this.classe = classe;
    }

    /**
     * Retorna el tipus genèric del paràmetre de l'acció
     * @return tipus genèric
     */
    public Type getParamGenericType() {
        return tipusgeneric;
    }

    /**
     * Defineix el tipus genèric del paràmetre de l'acció
     * @param tipusgeneric Tipus genèric
     */
    public void setParamGenericType(Type tipusgeneric) {
        this.tipusgeneric = tipusgeneric;
    }
    
}
